"use strict";

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

Object.defineProperty(exports, "__esModule", {
  value: true
});

var power_di_1 = require("power-di");

var utils_1 = require("power-di/utils");

var event_1 = require("../event");

var utils_2 = require("../utils");

var OduxConfig_1 = require("./OduxConfig");

var BaseStore_1 = require("./BaseStore");

var utils_3 = require("../utils");

var immer_1 = require("immer");

var ReduxListener_1 = require("./ReduxListener");

var utils_4 = require("../utils");

var getUid = function () {
  var id = 0;
  return function () {
    return id++;
  };
}();

var Odux =
/*#__PURE__*/
function () {
  function Odux(config) {
    var _this = this;

    _classCallCheck(this, Odux);

    this.config = config;
    this.debug = new utils_3.Debug();
    this.localStore = {};
    this.isTracking = 0;

    this.storeChangeListener = function (evt) {
      var state = evt.data;

      _this.debug.log('[storeChangeListener]', state);

      var changedStore = [];
      Object.keys(_this.localStore).forEach(function (k) {
        var storePath = _this.localStore[k].paths;

        try {
          var store = _this.localStore[k];
          var newValue = storePath.reduce(function (s, k) {
            return s[k];
          }, state);

          if (!utils_4.shallowEqual(store.value, newValue)) {
            changedStore.push({
              storeKey: k,
              value: newValue
            });
            store.value = newValue;
          }
        } catch (error) {
          _this.debug.info('[storeChangeListener] store: [', storePath.join('.'), '] not exist.', error);
        }
      });

      _this.eventBus.emitStoreChangeEvent(new event_1.StoreChangeEvent(changedStore));
    };

    this.config = config = Object.assign({}, new OduxConfig_1.OduxConfig(), this.config || {});

    if (!config.iocContext) {
      this.debug.warn('no iocContext in config, use IocContext.DefaultInstance.');
      config.iocContext = power_di_1.IocContext.DefaultInstance;
    } // register to iocContext


    if (config.iocContext.get(Odux)) {
      this.debug.warn('is already has [Odux] in IocContext, this new instance CANNOT register.');
    } else {
      config.iocContext.register(this, Odux);
    } // configure event bus


    this.eventBus = this.ioc.get(event_1.EventBus);

    if (!this.eventBus) {
      this.eventBus = new event_1.EventBus();
      this.ioc.register(this.eventBus, event_1.EventBus);
    }

    this.changeDispatch = new ReduxListener_1.ReduxListener(this.eventBus);
    this.eventBus.addListener(ReduxListener_1.ReduxChangeEvent, this.storeChangeListener);
    immer_1.setAutoFreeze(this.config.devMode);
    this.debug.isDebug = this.config.isDebug;
  }

  _createClass(Odux, [{
    key: "dispose",
    value: function dispose() {
      this.changeDispatch.unsubscribeReduxStore();
    }
  }, {
    key: "setReduxStore",
    value: function setReduxStore(store) {
      this.reduxStore = store;
      this.changeDispatch.change(store);
      return this;
    }
  }, {
    key: "getReduxStore",
    value: function getReduxStore() {
      return this.reduxStore;
    }
  }, {
    key: "initStores",
    value: function initStores(StoreTypes) {
      var _this2 = this;

      var ioc = this.ioc;

      if (!StoreTypes) {
        StoreTypes = ioc.getSubClasses(BaseStore_1.BaseStore) || [];
      }

      StoreTypes.forEach(function (StoreType) {
        if (utils_1.isClass(StoreType)) {
          ioc.replace(StoreType, new StoreType(_this2));
        }
      });
    }
  }, {
    key: "registerStore",
    value: function registerStore(store) {
      var StoreType = store.constructor;
      this.debug.log('[registerStore]', utils_1.getGlobalType(StoreType));

      if (!this.ioc.has(StoreType)) {
        this.ioc.register(store, StoreType);
      }
    }
  }, {
    key: "getStore",
    value: function getStore(type) {
      return this.ioc.get(type);
    }
  }, {
    key: "registerStorePath",
    value: function registerStorePath(storePath, oldStoreKey) {
      if (oldStoreKey) {
        delete this.localStore[oldStoreKey];
      }

      var storeKey = storePath.join('.');
      storeKey = this.localStore[storeKey] ? "".concat(storeKey, "_").concat(getUid()) : storeKey;
      this.localStore[storeKey] = {
        paths: storePath,
        value: this.reduxStore ? storePath.reduce(function (s, k) {
          return s[k] || {};
        }, this.reduxStore.getState()) : {},
        draft: undefined,
        inProduce: false
      };
      this.debug.log('[registerStorePath]', storePath, storeKey, oldStoreKey);
      return storeKey;
    }
  }, {
    key: "getStoreData",
    value: function getStoreData(storeKey) {
      if (!this.localStore[storeKey]) {
        throw new Error("No Store Register: [".concat(storeKey, "]"));
      }

      var store = this.localStore[storeKey];

      if (store.inProduce && !store.draft) {
        store.draft = immer_1.createDraft(store.value);
      }

      return store.draft || store.value;
    }
  }, {
    key: "transactionChange",
    value: function transactionChange(storeKey, func, err) {
      var _this3 = this;

      this.debug.log('[transactionBegin]', this.isTracking);
      this.isTracking++;
      this.debug.log('[transactionChange]', storeKey);
      var state = this.getStoreData(storeKey);
      this.debug.log('[before draft]', storeKey, state);
      var store = this.localStore[storeKey];
      var isFinish = false;

      var finish = function finish() {
        if (isFinish) {
          _this3.debug.warn('[finish 2rd]');

          return;
        }

        isFinish = true;

        if (_this3.isTracking > 0) {
          _this3.isTracking--;
        } else {
          _this3.debug.warn('[transactionEnd] tracking already is 0.');
        }

        if (storeFirstDraft) {
          store.inProduce = false;
          store.draft = store.draft && immer_1.finishDraft(store.draft);

          if (utils_4.shallowEqual(store.value, store.draft)) {
            store.draft = undefined;
          }
        }

        _this3.debug.log('[after draft]', storeKey, store.draft);

        _this3.debug.log('[transactionEnd]', _this3.isTracking);

        if (_this3.isTracking === 0) {
          _this3.dispatchChange();
        }
      };

      var storeFirstDraft = false;

      if (!store.inProduce) {
        store.draft = immer_1.createDraft(state);
        store.inProduce = true;
        storeFirstDraft = true;
      }

      var result;
      utils_2.guard(function () {
        result = func();
      }, undefined, function (error) {
        _this3.debug.error('transactionChange error', error);

        err && err(error);
      });

      if (result instanceof Promise) {
        return result.then(function (data) {
          finish();
          return data;
        }).catch(function (err) {
          finish();
          throw err;
        });
      } else {
        finish();
        return result;
      }
    }
  }, {
    key: "applyChange",
    value: function applyChange(storeKeys) {
      var _this4 = this;

      storeKeys.filter(function (k) {
        return _this4.localStore[k].inProduce && _this4.localStore[k].draft;
      }).forEach(function (k) {
        var store = _this4.localStore[k];
        store.draft = immer_1.finishDraft(store.draft);
      });
      this.dispatchAction(storeKeys);
    }
  }, {
    key: "mainReducer",
    value: function mainReducer(state, action) {
      var _this5 = this;

      var newState = Object.assign({}, state || {});
      Object.keys(this.localStore).forEach(function (key) {
        var store = _this5.localStore[key];

        if (store.paths.reduce(function (s, k) {
          return s && s[k];
        }, newState) === undefined) {
          _this5.setValue(newState, store.paths, store.value);
        }
      });

      if (action.type !== Odux.REDUX_ACTION_TYPE) {
        return newState;
      }

      this.debug.log('[mainReducer]', action);
      action.data.forEach(function (change) {
        _this5.setValue(newState, change.paths, change.newValue);
      });
      this.debug.log('[mainReducer] newState', newState);

      if (this.config.isDebug) {// compare(state, newState, '');
      }

      return newState;
    }
  }, {
    key: "setValue",
    value: function setValue(state, paths, value) {
      var prefix = _toConsumableArray(paths);

      var name = prefix.pop();
      prefix.forEach(function (k) {
        if (!state[k]) {
          state[k] = {};
        }

        state[k] = Object.assign({}, state[k]);
        state = state[k];
      });
      state[name] = value;
    }
  }, {
    key: "dispatchChange",
    value: function dispatchChange() {
      var _this6 = this;

      this.debug.log('[dispatchChange]', !!this.dispatchTimer);

      if (this.dispatchTimer) {
        return;
      }

      if (this.config.dispatchDelay === -1) {
        this.dispatchAction();
      } else {
        this.dispatchTimer = setTimeout(function () {
          _this6.dispatchTimer = undefined;

          _this6.dispatchAction();
        }, this.config.dispatchDelay);
      }
    }
  }, {
    key: "dispatchAction",
    value: function dispatchAction(storeKeys) {
      var _this7 = this;

      storeKeys = storeKeys || Object.keys(this.localStore);
      var changes = storeKeys.filter(function (k) {
        var store = _this7.localStore[k];
        return store.draft && !utils_4.shallowEqual(store.value, store.draft);
      }).map(function (k) {
        var store = _this7.localStore[k];
        var data = {
          storeKey: k,
          paths: store.paths,
          newValue: store.draft
        };
        store.draft = undefined;
        return data;
      });
      this.debug.log('[dispatchAction]', changes);

      if (changes.length) {
        if (this.reduxStore) {
          this.reduxStore.dispatch({
            type: Odux.REDUX_ACTION_TYPE,
            data: changes
          });
        } else {
          changes.forEach(function (change) {
            _this7.localStore[change.storeKey].value = change.newValue;
          });
          this.eventBus.emitStoreChangeEvent(new event_1.StoreChangeEvent(changes.map(function (change) {
            return {
              storeKey: change.storeKey,
              value: change.newValue
            };
          })));
        }
      }
    }
  }, {
    key: "ioc",
    get: function get() {
      return this.config.iocContext;
    }
  }], [{
    key: "REDUX_ACTION_TYPE",
    get: function get() {
      return '$$Odux';
    }
  }]);

  return Odux;
}();

exports.Odux = Odux;