"use strict";

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});

var core_1 = require("../core");

function mergeConfig(config) {
  return Object.assign({}, new core_1.OduxConfig(), config || {});
}

function createOdux(config, reduxStore) {
  config = mergeConfig(config);
  var odux = new core_1.Odux(config);

  if (reduxStore) {
    odux.setReduxStore(reduxStore);
  }

  odux.initStores();
  return odux;
}

exports.createOdux = createOdux;

var redux_1 = require("redux");

function createOduxAIO(config) {
  var middlewares = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var enhancers = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  config = mergeConfig(config);
  var odux = new core_1.Odux(config);
  var composeEnhancers = config.devMode && (typeof window === "undefined" ? "undefined" : _typeof(window)) === 'object' && window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ ? window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__({}) : redux_1.compose;
  var enhancer = composeEnhancers.apply(void 0, [redux_1.applyMiddleware.apply(redux_1, _toConsumableArray(middlewares))].concat(_toConsumableArray(enhancers)));
  var store = redux_1.createStore(odux.mainReducer.bind(odux), enhancer);
  odux.setReduxStore(store);
  odux.initStores();
  return odux;
}

exports.createOduxAIO = createOduxAIO;

function combineReducer(odux, reducer) {
  return function (state, action) {
    return odux.mainReducer(reducer(state, action), action);
  };
}

function createOduxEnhancer(config) {
  var odux = createOdux(config);

  if (config.devMode) {
    window._$Odux = odux;
  }

  return function (createStore) {
    return function (reducer, preloadedState, enhancer) {
      var rootReducer = combineReducer(odux, reducer);
      var store = createStore(rootReducer, preloadedState, enhancer);
      odux.setReduxStore(store);
      return store;
    };
  };
}

exports.createOduxEnhancer = createOduxEnhancer;

function createOduxForDva(config) {
  var odux = createOdux(config);
  return {
    odux: odux,
    extraEnhancers: function extraEnhancers(createStore) {
      return function (reducer, preloadedState, enhancer) {
        var store = createStore(reducer, preloadedState, enhancer);
        odux.setReduxStore(store);
        return store;
      };
    },
    onReducer: function onReducer(reducer) {
      return combineReducer(odux, reducer);
    }
  };
}

exports.createOduxForDva = createOduxForDva;