"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

require("reflect-metadata");

var utils_1 = require("power-di/utils");

var core_1 = require("../core");

var OduxMetaSymbol = Symbol('OduxMeta');
exports.PropsKeySymbol = '__$odux';

function getMetadata(target) {
  var data = Object.getOwnPropertyDescriptor(target, OduxMetaSymbol);

  if (!data) {
    Object.defineProperty(target, OduxMetaSymbol, {
      configurable: false,
      writable: false,
      value: {
        connect: {}
      }
    });
    return getMetadata(target);
  }

  return data.value;
}

exports.getMetadata = getMetadata;

function inject() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return function (target, key) {
    var meta = getMetadata(target.constructor);

    meta.connect[key] = config.getter || function (ioc) {
      var DataType = utils_1.getClsTypeByDecorator(config.dataType, target, key);

      if (utils_1.getSuperClassInfo(DataType).every(function (t) {
        return t.class !== core_1.BaseStore;
      })) {
        throw new Error("[odux]inject Type is not extend BaseStore: ".concat(DataType, " ").concat(target, " ").concat(key));
      }

      if (!ioc.has(DataType)) {
        if (ioc.has(core_1.Odux)) {
          ioc.register(DataType);
        } else {
          ioc.register(DataType, undefined, {
            autoNew: false,
            regInSuperClass: true
          });
        }
      }

      meta.connect[key] = function (ioc) {
        return ioc.get(DataType);
      };

      return meta.connect[key](ioc);
    };

    return {
      get: function get() {
        return this.props[exports.PropsKeySymbol][key];
      }
    };
  };
}

exports.inject = inject;